#!/usr/bin/env python3
"""
souleyez.ui - User interface components
"""
from .interactive import run_interactive_menu
from .dashboard import run_dashboard

__all__ = ['run_interactive_menu', 'run_dashboard']
